/* $Id: ddk.h,v 1.15 2000/06/12 14:52:05 ekohl Exp $
 *
 * COPYRIGHT:                See COPYING in the top level directory
 * PROJECT:                  ReactOS kernel
 * FILE:                     include/internal/hal/ddk.h
 * PURPOSE:                  HAL provided defintions for device drivers
 * PROGRAMMER:               David Welch (welch@mcmail.com)
 * REVISION HISTORY:
 *              23/06/98:   Taken from linux system.h
 */


#ifndef __INCLUDE_INTERNAL_HAL_DDK_H
#define __INCLUDE_INTERNAL_HAL_DDK_H

#include <internal/ntoskrnl.h>

/* HalReturnToFirmware */
#define FIRMWARE_HALT   1
#define FIRMWARE_REBOOT 3


enum
{
   DEVICE_DESCRIPTION_VERSION,
   DEVICE_DESCRIPTION_VERSION1,
};

typedef ULONG DMA_WIDTH;
typedef ULONG DMA_SPEED;

/*
 * PURPOSE: Types for HalGetBusData
 */
typedef enum _BUS_DATA_TYPE
{
   ConfigurationSpaceUndefined = -1,
   Cmos,
   EisaConfiguration,
   Pos,
   CbusConfiguration,
   PCIConfiguration,
   VMEConfiguration,
   NuBusConfiguration,
   PCMCIAConfiguration,
   MPIConfiguration,
   MPSAConfiguration,
   PNPISAConfiguration,
   MaximumBusDataType,
} BUS_DATA_TYPE, *PBUS_DATA_TYPE;

typedef struct _DEVICE_DESCRIPTION
{
   ULONG Version;
   BOOLEAN Master;
   BOOLEAN ScatterGather;
   BOOLEAN DemandMode;
   BOOLEAN AutoInitialize;
   BOOLEAN Dma32BitAddress;
   BOOLEAN IgnoreCount;
   BOOLEAN Reserved1;
   BOOLEAN Reserved2;
   ULONG BusNumber;
   ULONG DmaChannel;
   INTERFACE_TYPE InterfaceType;
   DMA_WIDTH DmaWidth;
   DMA_SPEED DmaSpeed;
   ULONG MaximumLength;
   ULONG DmaPort;
} DEVICE_DESCRIPTION, *PDEVICE_DESCRIPTION;

typedef BOOLEAN (*PHAL_RESET_DISPLAY_PARAMETERS)(ULONG Columns, ULONG Rows);


VOID
STDCALL
HalAcquireDisplayOwnership (
	PHAL_RESET_DISPLAY_PARAMETERS	ResetDisplayParameters
	);

NTSTATUS
STDCALL
HalAdjustResourceList (
	PCM_RESOURCE_LIST	Resources
	);

NTSTATUS
STDCALL
HalAllocateAdapterChannel (
	IN	PADAPTER_OBJECT	AdapterObject,
	ULONG	Unknown2,
	ULONG	Unknown3,
	ULONG	Unknown4
	);

PVOID
STDCALL
HalAllocateCommonBuffer (
	PADAPTER_OBJECT		AdapterObject,
	ULONG			Length,
	PPHYSICAL_ADDRESS	LogicalAddress,
	BOOLEAN			CacheEnabled
	);

PVOID
STDCALL
HalAllocateCrashDumpRegisters (
	IN	PADAPTER_OBJECT	AdapterObject,
	IN OUT	PULONG		NumberOfMapRegisters
	);

BOOLEAN
STDCALL
HalAllProcessorsStarted (
	VOID
	);

NTSTATUS
STDCALL
HalAssignSlotResources (
	PUNICODE_STRING		RegistryPath,
	PUNICODE_STRING		DriverClassName,
	PDRIVER_OBJECT		DriverObject,
	PDEVICE_OBJECT		DeviceObject,
	INTERFACE_TYPE		BusType,
	ULONG			BusNumber,
	ULONG			SlotNumber,
	PCM_RESOURCE_LIST	*AllocatedResources
	);

/*
HalBeginSystemInterrupt
*/

/*
HalCalibratePerformanceCounter
*/

/*
FASTCALL
HalClearSoftwareInterrupt
*/

/*
HalDisableSystemInterrupt
*/

VOID
STDCALL
HalDisplayString (
	IN	PCH	String
	);

/*
HalEnableSystemInterrupt
*/

/*
HalEndSystemInterrupt
*/

/* Is this function really exported ?? */
VOID
HalExamineMBR (
	PDEVICE_OBJECT	DeviceObject,
	ULONG		SectorSize,
	ULONG		MBRTypeIdentifier,
	PVOID		Buffer
	);

BOOLEAN
STDCALL
HalFlushCommonBuffer (
	ULONG	Unknown1,
	ULONG	Unknown2,
	ULONG	Unknown3,
	ULONG	Unknown4,
	ULONG	Unknown5,
	ULONG	Unknown6,
	ULONG	Unknown7,
	ULONG	Unknown8
	);

VOID
STDCALL
HalFreeCommonBuffer (
	PADAPTER_OBJECT		AdapterObject,
	ULONG			Length,
	PHYSICAL_ADDRESS	LogicalAddress,
	PVOID			VirtualAddress,
	BOOLEAN			CacheEnabled
	);

PADAPTER_OBJECT
STDCALL
HalGetAdapter (
	PDEVICE_DESCRIPTION	DeviceDescription,
	PULONG			NumberOfMapRegisters
	);

ULONG
STDCALL
HalGetBusData (
	BUS_DATA_TYPE	BusDataType,
	ULONG		BusNumber,
	ULONG		SlotNumber,
	PVOID		Buffer,
	ULONG		Length
	);

ULONG
STDCALL
HalGetBusDataByOffset (
	BUS_DATA_TYPE	BusDataType,
	ULONG		BusNumber,
	ULONG		SlotNumber,
	PVOID		Buffer,
	ULONG		Offset,
	ULONG		Length
	);

/* Is this function really exported ?? */
ULONG
HalGetDmaAlignmentRequirement (
	VOID
	);

/*
HalGetEnvironmentVariable
*/

ULONG
STDCALL
HalGetInterruptVector (
	INTERFACE_TYPE	InterfaceType,
	ULONG		BusNumber,
	ULONG		BusInterruptLevel,
	ULONG		BusInterruptVector,
	PKIRQL		Irql,
	PKAFFINITY	Affinity
	);

VOID
STDCALL
HalInitializeProcessor (
	ULONG	ProcessorNumber
	);

BOOLEAN
STDCALL
HalInitSystem (
	ULONG		Phase,
	boot_param	*bp
	);

BOOLEAN
STDCALL
HalMakeBeep (
	ULONG Frequency
	);

VOID
STDCALL
HalQueryDisplayParameters (
	PULONG	DispSizeX,
	PULONG	DispSizeY,
	PULONG	CursorPosX,
	PULONG	CursorPosY
	);

VOID
STDCALL
HalQueryRealTimeClock (
	PTIME_FIELDS	Time
	);

/* Is this function really exported ?? */
VOID
HalQuerySystemInformation(VOID);

ULONG
STDCALL
HalReadDmaCounter (
	PADAPTER_OBJECT	AdapterObject
	);

VOID
STDCALL
HalReportResourceUsage (
	VOID
	);

VOID
STDCALL
HalRequestIpi (
	ULONG	Unknown
	);

/*
FASTCALL
HalRequestSoftwareInterrupt
*/

VOID
STDCALL
HalReturnToFirmware (
	ULONG	Action
	);

ULONG
STDCALL
HalSetBusData (
	BUS_DATA_TYPE	BusDataType,
	ULONG		BusNumber,
	ULONG		SlotNumber,
	PVOID		Buffer,
	ULONG		Length
	);

ULONG
STDCALL
HalSetBusDataByOffset (
	BUS_DATA_TYPE	BusDataType,
	ULONG		BusNumber,
	ULONG		SlotNumber,
	PVOID		Buffer,
	ULONG		Offset,
	ULONG		Length
	);

VOID
STDCALL
HalSetDisplayParameters (
	ULONG	CursorPosX,
	ULONG	CursorPosY
	);

/*
HalSetEnvironmentVariable
*/

/*
HalSetProfileInterval
*/

VOID
STDCALL
HalSetRealTimeClock (
	PTIME_FIELDS	Time
	);

/*
HalSetTimeIncrement
*/

BOOLEAN
STDCALL
HalStartNextProcessor (
	ULONG	Unknown1,
	ULONG	Unknown2
	);

/*
HalStartProfileInterrupt
*/

/*
HalStopProfileInterrupt
*/

ULONG
FASTCALL
HalSystemVectorDispatchEntry (
	ULONG	Unknown1,
	ULONG	Unknown2,
	ULONG	Unknown3
	);

BOOLEAN
STDCALL
HalTranslateBusAddress (
	INTERFACE_TYPE		InterfaceType,
	ULONG			BusNumber,
	PHYSICAL_ADDRESS	BusAddress,
	PULONG			AddressSpace,
	PPHYSICAL_ADDRESS	TranslatedAddress
	);

/*
 * Kernel debugger section
 */

typedef struct _KD_PORT_INFORMATION
{
	ULONG ComPort;
	ULONG BaudRate;
	ULONG BaseAddress;
} KD_PORT_INFORMATION, *PKD_PORT_INFORMATION;


#if defined(__HAL__) || defined(__NTOSKRNL__)
extern ULONG KdComPortInUse __declspec(dllexport);
#else
extern ULONG KdComPortInUse __declspec(dllimport);
#endif

BOOLEAN
STDCALL
KdPortInitialize (PKD_PORT_INFORMATION PortInformation,
		  DWORD Unknown1,
		  DWORD Unknown2);

BOOLEAN
STDCALL
KdPortGetByte (PUCHAR ByteRecieved);

BOOLEAN
STDCALL
KdPortPollByte (PUCHAR ByteRecieved);

VOID
STDCALL
KdPortPutByte (UCHAR ByteToSend);


/*
 * Port I/O functions
 */

VOID
STDCALL
READ_PORT_BUFFER_UCHAR (PUCHAR Port, PUCHAR Value, ULONG Count);

VOID
STDCALL
READ_PORT_BUFFER_ULONG (PULONG Port, PULONG Value, ULONG Count);

VOID
STDCALL
READ_PORT_BUFFER_USHORT (PUSHORT Port, PUSHORT Value, ULONG Count);

UCHAR
STDCALL
READ_PORT_UCHAR (PUCHAR Port);

ULONG
STDCALL
READ_PORT_ULONG (PULONG Port);

USHORT
STDCALL
READ_PORT_USHORT (PUSHORT Port);

VOID
STDCALL
WRITE_PORT_BUFFER_UCHAR (PUCHAR Port, PUCHAR Value, ULONG Count);

VOID
STDCALL
WRITE_PORT_BUFFER_ULONG (PULONG Port, PULONG Value, ULONG Count);

VOID
STDCALL
WRITE_PORT_BUFFER_USHORT (PUSHORT Port, PUSHORT Value, ULONG Count);

VOID
STDCALL
WRITE_PORT_UCHAR (PUCHAR Port, UCHAR Value);

VOID
STDCALL
WRITE_PORT_ULONG (PULONG Port, ULONG Value);

VOID
STDCALL
WRITE_PORT_USHORT (PUSHORT Port, USHORT Value);


#endif /* __INCLUDE_INTERNAL_HAL_DDK_H */
